@@ -1,5 +1,6 @@
 package Amazon::MWS::Uploader;
 
+use utf8;
 use strict;
 use warnings;
 
@@ -22,7 +23,7 @@ use Moo;
 use MooX::Types::MooseLike::Base qw(:all);
 use namespace::clean;
 
-our $VERSION = '0.14';
+our $VERSION = '0.16';
 
 use constant {
     AMW_ORDER_WILDCARD_ERROR => 999999,
@@ -78,6 +79,10 @@ When migrating from 0.05 to 0.06 please execute this SQL statement
  ALTER TABLE amazon_mws_products ADD COLUMN listed BOOLEAN;
  UPDATE amazon_mws_products SET listed = 1 WHERE status = 'ok';
 
+When upgrading to 0.16, please execute this SQL statement:
+
+ ALTER TABLE amazon_mws_products ADD COLUMN warnings TEXT;
+
 =head1 ACCESSORS
 
 The following keys must be passed at the constructor and can be
@@ -618,11 +623,11 @@ has _mismatch_patterns => (is => 'lazy', isa => HashRef);
 
 sub _build__mismatch_patterns {
     my $self = shift;
-    my $merchant_re = qr{\s+\(Merchant:\s+'(.*?)'\s+/};
+    my $merchant_re = qr{\s+\((?:Merchant|Verkäufer):\s+'(.*?)'\s+/};
     my $amazon_re = qr{\s+.*?/\s*Amazon:\s+'(.*?)'\)};
     my %patterns = (
                     # informative only
-                    asin => qr{ASIN\s+([0-9A-Za-z]+)},
+                    asin => qr{ASIN(?:\s+überein)?\s+([0-9A-Za-z]+)},
 
                     shop_part_number => qr{part_number$merchant_re},
                     amazon_part_number => qr{part_number$amazon_re},
@@ -635,6 +640,13 @@ sub _build__mismatch_patterns {
 
                     shop_brand => qr{brand$merchant_re},
                     amazon_brand => qr{brand$amazon_re},
+
+                    shop_color => qr{color$merchant_re},
+                    amazon_color => qr{color$amazon_re},
+
+                    shop_size => qr{size$merchant_re},
+                    amazon_size => qr{size$amazon_re},
+
                    );
     return \%patterns;
 }
@@ -757,6 +769,7 @@ sub _mark_products_as_pending {
         my %data = (
                     amws_job_id => $job_id,
                     status => 'pending',
+                    warnings => '', # clear out
                     timestamp_string => $p->timestamp_string,
                    );
         my $check = $self
@@ -1105,6 +1118,15 @@ sub upload_feed {
                     foreach my $w (@$warns) {
                         $self->_error_logger(warning => $w->{code},
                                              "$w->{sku}: $w->{error}");
+                        # and register it in the db
+                        if ($w->{sku} && $w->{error}) {
+                            $self->_exe_query($self->sqla->update('amazon_mws_products',
+                                                                  { warnings => "$job_id $w->{code} $w->{error}" },
+                                                                  {
+                                                                   sku => $w->{sku},
+                                                                   shop_id => $self->_unique_shop_id,
+                                                                  }));
+                        }
                     }
                 }
                 else {
@@ -2339,6 +2361,11 @@ Return a list of hashref with the rows from C<amazon_mws_products> for
 the current shop and the error code passed as argument. If no error
 codes are passed, fetch all the products in error.
 
+=head2 get_products_with_warnings
+
+Returns a list of hashref, with C<sku> and C<warnings> as keys, for
+each product in the shop which has the warnings set to something.
+
 =cut
 
 sub get_products_with_error_code {
@@ -2362,6 +2389,23 @@ sub get_products_with_error_code {
     return @records;
 }
 
+sub get_products_with_warnings {
+    my $self = shift;
+    my $sth = $self->_exe_query($self->sqla
+                                ->select('amazon_mws_products', '*',
+                                         {
+                                          status => 'ok',
+                                          shop_id => $self->_unique_shop_id,
+                                          warnings => { '!=' => '' },
+                                         },
+                                         [qw/sku warnings/]));
+    my @records;
+    while (my $row = $sth->fetchrow_hashref) {
+        push @records, $row;
+    }
+    return @records;
+}
+
 =head2 mark_failed_products_as_redo(@skus)
 
 Alter the status of the failed skus passed as argument from 'failed'
@@ -2390,7 +2434,7 @@ the failed skus, and the values are hashrefs where the keys are the
 mismatched fields and the values are hashrefs with these keys:
 
 Mismatched fields may be: C<part_number>, C<title>, C<manufacturer>,
-C<brand>.
+C<brand>, C<color>, C<size>
 
 =over 4
 
